import { JsonClientResponse } from "../client/JsonClientResponse";
import { DialogProxyTools } from "../proxy/DialogProxyTools";
import { SessionState } from "../proxy/SessionState";
import { Log } from '../util/Log';
import { SdaDialogDelegateTools } from "./SdaDialogDelegateTools";
export class SdaDialogDelegate {
    constructor() {
        this._delegateState = null;
        this._lastActivity = new Date();
    }
    get lastActivity() {
        return this._lastActivity;
    }
    initialize() {
        Log.info("SdaDialogDelegate::initialize -- initializing");
        return Promise.resolve().then(voidValue => {
            Log.info("SdaDialogDelegate::initialize -- showing storage keys");
            return SdaDialogDelegateTools.showAllStorageKeys();
        }).then(voidValue => {
            return SdaDialogDelegateTools.readDelegateState();
        }).then(delegateState => {
            this._delegateState = delegateState;
            Log.info(`SdaDialogDelegate::initialize -- current delegate state: ${this._delegateState.copyAsJsonString()}`);
            Log.info("SdaDialogDelegate::initialize -- SdaDialogDelegate is done initializing");
        });
    }
    addWorkPackageToBriefcase(baseUrl, resourcePathElems, body) {
        // REQUIRED: Before we can add a Work Package to the briefcase, we must be online
        if (!this.online()) {
            const dialogMessage = DialogProxyTools.constructDialogMessageModel("Cannot add a Work Package to the briefcase while offline");
            return Promise.resolve(new JsonClientResponse(dialogMessage, 400));
        }
        Log.info("SdaDialogDelegate::addWorkpackageToBriefcase -- needs implementation: " + resourcePathElems);
        return null;
    }
    getBlob(baseUrl, resourcePath) {
        Log.info("SdaDialogDelegate::getBlob -- path: " + resourcePath);
        return null;
    }
    getText(baseUrl, resourcePath) {
        Log.info("SdaDialogDelegate::getText -- path: " + resourcePath);
        return null;
    }
    openStream(baseUrl, resourcePath) {
        Log.info("SdaDialogDelegate::openStream -- path: " + resourcePath);
        return null;
    }
    postMultipart(baseUrl, resourcePath, formData) {
        Log.info("SdaDialogDelegate::postMultipart -- path: " + resourcePath);
        return null;
    }
    getJson(baseUrl, resourcePath, queryParams) {
        Log.info("SdaDialogDelegate::getJson -- path: " + resourcePath);
        return null;
    }
    postJson(baseUrl, resourcePath, body) {
        Log.info("SdaDialogDelegate::postJson -- path: " + resourcePath);
        Log.info("SdaDialogDelegate::postJson -- body: " + JSON.stringify(body));
        const resourcePathElems = resourcePath.split('/');
        if (SdaDialogDelegateTools.isAddToBriefcaseMenuActionRequest(resourcePathElems)) {
            return this.addWorkPackageToBriefcase(baseUrl, resourcePathElems, body);
        }
        if (DialogProxyTools.isCreateSessionRequest(resourcePathElems)) {
            Log.info("SdaDialogDelegate::postJson -- CREATE SESSION");
        }
        return null;
    }
    putJson(baseUrl, resourcePath, body) {
        Log.info("SdaDialogDelegate::putJson -- path: " + resourcePath);
        Log.info("SdaDialogDelegate::putJson -- body: " + JSON.stringify(body));
        return null;
    }
    deleteJson(baseUrl, resourcePath) {
        Log.info("SdaDialogDelegate::deleteJson -- path: " + resourcePath);
        return null;
    }
    handleDeleteJsonResponse(baseUrl, resourcePath, response) {
        Log.info("SdaDialogDelegate::handleDeleteJsonResponse -- path: " + resourcePath);
        response.then(jcr => Log.info("SdaDialogDelegate::handleDeleteJsonResponse -- json response: " + JSON.stringify(jcr.value)));
        return response;
    }
    handleGetBlobResponse(baseUrl, resourcePath, response) {
        Log.info("SdaDialogDelegate::handleGetBlobResponse -- path: " + resourcePath);
        response.then(bcr => Log.info("SdaDialogDelegate::handleGetBlobResponse -- blob response: " + JSON.stringify(bcr.value)));
        return response;
    }
    handleGetJsonResponse(baseUrl, resourcePath, queryParams, response) {
        Log.info("SdaDialogDelegate::handleGetJsonResponse -- path: " + resourcePath);
        response.then(jcr => Log.info("SdaDialogDelegate::handleGetJsonResponse -- json response: " + JSON.stringify(jcr.value)));
        return response.then(jcr => {
            if (jcr.statusCode === 200) {
                const jsonObject = jcr.value;
                if (SdaDialogDelegateTools.isWorkPackagesRootDialog(jsonObject)) {
                    const resourcePathElems = resourcePath.split('/');
                    const pathFields = DialogProxyTools.deconstructGetDialogPath(resourcePathElems);
                    //                    DialogProxyTools.writeDialogState(pathFields.tenantId, pathFields.sessionId, )
                    const workPackagesDialog = SdaDialogDelegateTools.patchWorkPackagesDialog(jsonObject);
                    return new JsonClientResponse(workPackagesDialog, 200);
                }
            }
            return jcr;
        });
    }
    handleGetTextResponse(baseUrl, resourcePath, response) {
        Log.info("SdaDialogDelegate::handleGetTextResponse -- path: " + resourcePath);
        response.then(tcr => Log.info("SdaDialogDelegate::handleGetTextResponse -- text response: " + JSON.stringify(tcr.value)));
        return response;
    }
    handleOpenStreamResponse(baseUrl, resourcePath, response) {
        Log.info("SdaDialogDelegate::handleOpenStreamResponse -- path: " + resourcePath);
        response.then(sp => Log.info("SdaDialogDelegate::handleOpenStreamResponse -- stream producer response: " + sp));
        return response;
    }
    handlePostJsonResponse(baseUrl, resourcePath, body, response) {
        Log.info("SdaDialogDelegate::handlePostJsonResponse -- path: " + resourcePath);
        response.then(jcr => Log.info("SdaDialogDelegate::handlePostJsonResponse -- json response: " + JSON.stringify(jcr.value)));
        return response.then(jcr => {
            if (jcr.statusCode === 200) {
                const jsonObject = jcr.value;
                if (DialogProxyTools.isSessionRootDialog(jsonObject)) {
                    this._delegateState.setUserId(SessionState.userId(jsonObject));
                    return SdaDialogDelegateTools.writeDelegateState(this._delegateState).then(voidValue => {
                        return SdaDialogDelegateTools.showAllStorageKeys().then(voidValue2 => SdaDialogDelegateTools.readDelegateState()).then(stateValue => {
                            Log.info('SdaDialogDelegate::handlePostJsonResponse -- delegate state after write: ' + stateValue.copyAsJsonString());
                            return jcr;
                        });
                    });
                }
            }
            return jcr;
        });
    }
    handlePostMultipartResponse(baseUrl, resourcePath, formData, response) {
        Log.info("SdaDialogDelegate::handlePostMultipartResponse -- path: " + resourcePath);
        response.then(vcr => Log.info("SdaDialogDelegate::handlePostMultipartResponse -- void response: " + JSON.stringify(vcr.value)));
        return response;
    }
    handlePutJsonResponse(baseUrl, resourcePath, body, response) {
        Log.info("SdaDialogDelegate::handlePutJsonResponse -- path: " + resourcePath);
        return response;
    }
    online() {
        return this._delegateState.briefcaseRecordState().online();
    }
}
