import { DialogProxyTools } from "../proxy/DialogProxyTools";
import { storage } from "../storage";
import { Log } from "../util/Log";
import { SdaGetBriefcaseRecordJsonSample } from "./samples/SdaGetBriefcaseRecordJsonSample";
import { SdaDialogDelegateState } from "./SdaDialogDelegateState";
export class SdaDialogDelegateTools {
    static isAddToBriefcaseMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.ADD_TO_BRIEFCASE_MENU_ACTION_ID;
    }
    static isWorkPackagesWorkbenchActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostWorkbenchAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostWorkbenchActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.WORK_PACKAGES_WORKBENCH_ACTION_ID;
    }
    static isWorkPackagesRootDialog(dialog) {
        if (!dialog || !dialog.type || !dialog.dialogAlias) {
            return false;
        }
        return dialog.type === this.EDITOR_DIALOG_MODEL_TYPE &&
            dialog.dialogAlias === this.WORK_PACKAGES_ROOT_DIALOG_ALIAS;
    }
    static patchWorkPackagesDialog(originalDialog) {
        const workPackagesQueryDialog = originalDialog.children[0];
        const propertyDefs = workPackagesQueryDialog.recordDef.propertyDefs;
        propertyDefs.push({
            "writeAllowed": false,
            "propertyName": "briefcase",
            "canCauseSideEffects": false,
            "upperCaseOnly": false,
            "propertyType": "boolean",
            "type": "hxgn.api.dialog.PropertyDef",
            "writeEnabled": false
        });
        const columns = workPackagesQueryDialog.view.columns;
        columns.push({
            "propertyName": "briefcase",
            "heading": "Briefcase",
            "type": "hxgn.api.dialog.Column"
        });
        // Return original dialog WITH patches
        return originalDialog;
    }
    static patchWorkPackagesRecordSet(originalRecordSet, delegateRecordSet) {
        const delegateRecords = delegateRecordSet ? delegateRecordSet.records : [];
        const originalRecords = originalRecordSet.records;
        if (originalRecords) {
            for (const r of originalRecords) {
                let delegateBriefcase = false;
                for (const delegateRecord of delegateRecords) {
                    if (r.id === delegateRecord.id) {
                        delegateBriefcase = true;
                        break;
                    }
                }
                const briefcaseField = {
                    "name": "briefcase",
                    "annotations": [],
                    "type": "hxgn.api.dialog.Property",
                    "value": delegateBriefcase
                };
                r.properties.push(briefcaseField);
            }
        }
        // Return original record set with patches
        return originalRecordSet;
    }
    static readDelegateState() {
        return storage.getJson(SdaDialogDelegateTools.DIALOG_DELEGATE_STATE_KEY).then(jsonObject => {
            if (!jsonObject) {
                jsonObject = { briefcaseRecord: SdaGetBriefcaseRecordJsonSample.response(), userId: null };
                Log.info('SdaDialogDelegateTools::readDelegateState -- returning defaults: ' + JSON.stringify(jsonObject));
            }
            return new SdaDialogDelegateState(jsonObject);
        });
    }
    static showAllStorageKeys() {
        return storage.getAllKeys().then(allKeys => {
            const keyCount = allKeys.length;
            Log.info(`SdaDialogDelegateTools::showAllStorageKeys -- key count: ${allKeys.length}`);
            for (let i = keyCount - 1; i > -1; --i) {
                Log.info(`SdaDialogDelegateTools::showAllStorageKeys -- key[${i}]: ${allKeys[i]}`);
            }
        }).catch(allKeysError => {
            Log.error("SdaDialogDelegateTools::showAllStorageKeys -- error getting all keys from storage: " + allKeysError);
        });
    }
    static writeDelegateState(delegateState) {
        Log.info('SdaDialogDelegateTools::writeDelegateState -- delegate state: ' + delegateState.copyAsJsonString());
        return storage.setJson(SdaDialogDelegateTools.DIALOG_DELEGATE_STATE_KEY, delegateState.internalValue());
    }
}
// Action Ids
SdaDialogDelegateTools.ADD_TO_BRIEFCASE_MENU_ACTION_ID = 'alias_AddToBriefcase';
SdaDialogDelegateTools.WORK_PACKAGES_WORKBENCH_ACTION_ID = 'WorkPackages';
// Dialog Names
SdaDialogDelegateTools.WORK_PACKAGES_ROOT_DIALOG_ALIAS = "Workpackage_General_FORM";
// Model Types
SdaDialogDelegateTools.EDITOR_DIALOG_MODEL_TYPE = "hxgn.api.dialog.EditorDialog";
SdaDialogDelegateTools.QUERY_DIALOG_MODEL_TYPE = "hxgn.api.dialog.QueryDialog";
// Property Names
SdaDialogDelegateTools.ONLINE_PROPERTY_NAME = 'online';
// Storage Keys
SdaDialogDelegateTools.BRIEFCASE_RECORD_KEY = 'ppm.sda.briefcase.record';
SdaDialogDelegateTools.DIALOG_DELEGATE_STATE_KEY = 'ppm.sda.dialog.delegate.state';
