import { SdaDialogDelegate } from "../ppm/SdaDialogDelegate";
import { Log } from '../util/Log';
import { DialogProxyTools } from './DialogProxyTools';
export class DialogProxy {
    constructor() {
        this._lastActivity = new Date();
        this._dialogDelegateChain = [new SdaDialogDelegate()];
    }
    get lastActivity() {
        return this._lastActivity;
    }
    getBlob(baseUrl, resourcePath) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let response;
            for (const d of this._dialogDelegateChain) {
                response = d.getBlob(baseUrl, resourcePath);
                if (response !== null) {
                    break;
                }
            }
            if (!response) {
                response = DialogProxyTools.commonFetchClient().getBlob(baseUrl, resourcePath);
            }
            for (const d of this._dialogDelegateChain) {
                response = d.handleGetBlobResponse(baseUrl, resourcePath, response);
            }
            return response;
        });
    }
    getText(baseUrl, resourcePath) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let response;
            for (const d of this._dialogDelegateChain) {
                response = d.getText(baseUrl, resourcePath);
                if (response !== null) {
                    break;
                }
            }
            if (!response) {
                response = DialogProxyTools.commonFetchClient().getText(baseUrl, resourcePath);
            }
            for (const d of this._dialogDelegateChain) {
                response = d.handleGetTextResponse(baseUrl, resourcePath, response);
            }
            return response;
        });
    }
    openStream(baseUrl, resourcePath) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let response;
            for (const d of this._dialogDelegateChain) {
                response = d.openStream(baseUrl, resourcePath);
                if (response !== null) {
                    break;
                }
            }
            if (!response) {
                response = DialogProxyTools.commonFetchClient().openStream(baseUrl, resourcePath);
            }
            for (const d of this._dialogDelegateChain) {
                response = d.handleOpenStreamResponse(baseUrl, resourcePath, response);
            }
            return response;
        });
    }
    postMultipart(baseUrl, resourcePath, formData) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let response;
            for (const d of this._dialogDelegateChain) {
                response = d.postMultipart(baseUrl, resourcePath, formData);
                if (response !== null) {
                    break;
                }
            }
            if (!response) {
                response = DialogProxyTools.commonFetchClient().postMultipart(baseUrl, resourcePath, formData);
            }
            for (const d of this._dialogDelegateChain) {
                response = d.handlePostMultipartResponse(baseUrl, resourcePath, formData, response);
            }
            return response;
        });
    }
    getJson(baseUrl, resourcePath, queryParams) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let response;
            for (const d of this._dialogDelegateChain) {
                response = d.getJson(baseUrl, resourcePath, queryParams);
                if (response !== null) {
                    break;
                }
            }
            if (!response) {
                response = DialogProxyTools.commonFetchClient().getJson(baseUrl, resourcePath, queryParams);
            }
            for (const d of this._dialogDelegateChain) {
                response = d.handleGetJsonResponse(baseUrl, resourcePath, queryParams, response);
            }
            return response;
        });
    }
    postJson(baseUrl, resourcePath, jsonBody) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let response;
            for (const d of this._dialogDelegateChain) {
                response = d.postJson(baseUrl, resourcePath, jsonBody);
                if (response !== null) {
                    break;
                }
            }
            if (!response) {
                response = DialogProxyTools.commonFetchClient().postJson(baseUrl, resourcePath, jsonBody);
            }
            for (const d of this._dialogDelegateChain) {
                response = d.handlePostJsonResponse(baseUrl, resourcePath, jsonBody, response);
            }
            return response;
        });
    }
    putJson(baseUrl, resourcePath, jsonBody) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let reponse;
            for (const d of this._dialogDelegateChain) {
                reponse = d.putJson(baseUrl, resourcePath, jsonBody);
                if (reponse !== null) {
                    break;
                }
            }
            if (!reponse) {
                reponse = DialogProxyTools.commonFetchClient().putJson(baseUrl, resourcePath, jsonBody);
            }
            for (const d of this._dialogDelegateChain) {
                reponse = d.handlePutJsonResponse(baseUrl, resourcePath, jsonBody, reponse);
            }
            return reponse;
        });
    }
    deleteJson(baseUrl, resourcePath) {
        this.prepareForActivity();
        return this._initialized.then(() => {
            let response;
            for (const d of this._dialogDelegateChain) {
                response = d.deleteJson(baseUrl, resourcePath);
                if (response !== null) {
                    break;
                }
            }
            if (!response) {
                response = DialogProxyTools.commonFetchClient().deleteJson(baseUrl, resourcePath);
            }
            for (const d of this._dialogDelegateChain) {
                response = d.handleDeleteJsonResponse(baseUrl, resourcePath, response);
            }
            return response;
        });
    }
    prepareForActivity() {
        this._lastActivity = new Date();
        if (!this._initialized) {
            Log.info("DialogProxy::prepareForActivity -- waiting for all DialogDelegates to initialize");
            const allDelegatesInitializing = this._dialogDelegateChain.map(d => d.initialize());
            this._initialized = Promise.all(allDelegatesInitializing).then(() => {
                Log.info("DialogProxy::prepareForActivity -- all DialogDelegates are initialized");
                Log.info("DialogProxy::prepareForActivity -- DialogProxy is initialized");
                return true;
            });
        }
    }
}
